/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIUtil.java 16:03:11 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import javax.naming.Context;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

/**
 * This utility class is used to help us to manage JNDI directories
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public final class JNDIUtil {

    private JNDIUtil() {

    }

    /**
     * Check if a name is bound is a specific context
     * 
     * @param parentContext
     *            parent context where to list the content of the child context
     * @param childContextName
     *            name of the child context, the context in we want to look for
     *            the binding name
     * @param name
     *            binding name
     * @return
     */
    public static boolean isBound(Context parentContext,
        String childContextName, String name) {
        boolean test = false;
        try {
            NamingEnumeration<NameClassPair> namingEnumeration = parentContext
                .list(childContextName);
            while (namingEnumeration.hasMoreElements()) {
                NameClassPair classPair = (NameClassPair) namingEnumeration
                    .next();
                if (classPair.getName().equals(name)) {
                    return true;
                }
            }
        } catch (NamingException e) {
            return false;
        }
        return test;
    }
}
