package org.objectweb.petals.util;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class is used to format various logs. it can be used as a static class
 * or as a wrapper object of a logger. The methods add to the resulting message
 * a "ClassName-MethodName" information. (only for DEBUG and INFO level)
 * 
 * @author alouis
 * 
 */
public class LoggingUtil {

    private Logger logger;

    public LoggingUtil(Logger logger) {
        this.logger = logger;
    }

    private LoggingUtil() {
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> : [CALL ]
     * 
     * @param log
     */
    public static void call(Logger log) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_DEBUG, "[CALL ] " + classAndMethod());
        }
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> : [CALL ]
     * 
     * @param log
     * @param msg
     */
    public static void call(Logger log, Object msg) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_DEBUG, "[CALL ] " + classAndMethod() + " "
                + msg);
        }
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message
     * 
     * @param log
     * @param msg
     */
    public static void debug(Logger log, Object message) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_DEBUG, classAndMethod() + " " + message);
        }
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> : [ END ]
     * 
     * @param log
     */
    public static void end(Logger log) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_DEBUG, "[ END ] " + classAndMethod());
        }
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message : [ END ]msg
     * 
     * @param log
     * @param msg
     */
    public static void end(Logger log, Object msg) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_DEBUG, "[ END ] " + classAndMethod() + " "
                + msg);
        }
    }

    /**
     * if log is enabled, Log as <i>ERROR</i> the message
     * 
     * @param log
     * @param msg
     */
    public static void error(Logger log, Object message) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_ERROR, message);
        }
    }

    /**
     * if log is enabled, Log as <i>ERROR</i> the exception
     * 
     * @param log
     * @param msg
     * @param throwable
     */
    public static void error(Logger log, Object message, Throwable throwable) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_ERROR, message, throwable);
        }
    }

    /**
     * if log is enabled, Log as <i>INFO</i> the message
     * 
     * @param log
     * @param msg
     */
    public static void info(Logger log, Object message) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_INFO, classAndMethod() + " " + message);
        }
    }

    /**
     * if log is enabled, Log as <i>INFO</i> the message and error
     * 
     * @param log
     * @param msg
     * @param error
     */
    public static void info(Logger log, Object message, Throwable error) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_INFO, classAndMethod() + " " + message,
                error);
        }
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> : [START]
     * 
     * @param log
     */
    public static void start(Logger log) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_DEBUG, "[START] " + classAndMethod());
        }
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message : [START]msg
     * 
     * @param log
     * @param msg
     */
    public static void start(Logger log, Object msg) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_DEBUG, "[START] " + classAndMethod() + " "
                + msg);
        }
    }

    /**
     * if log is enabled, Log as warning the message
     * 
     * @param log
     * @param msg
     */
    public static void warning(Logger log, Object message) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_WARN, message);
        }
    }

    /**
     * if log is enabled, Log as warning the exception
     * 
     * @param log
     * @param msg
     * @param throwable
     */
    public static void warning(Logger log, Object message, Throwable throwable) {
        if (log != null && log.isOn()) {
            log.log(BasicLevel.LEVEL_WARN, message, throwable);
        }
    }

    // //////////////////////////////////////////////////////////
    // Static part
    // //////////////////////////////////////////////////////////

    /**
     * Create an exception and analyse it in order to find the class and method
     * that called the LoggingUtil method.
     * 
     * @return
     */
    private static String classAndMethod() {
        String result = null;

        // throw and Parse an exception to find in the stack
        // the method that called the Loggingutil

        Throwable t = new Throwable().fillInStackTrace();

        StackTraceElement[] ste = t.getStackTrace();

        if (ste != null && ste.length > 2) {
            StackTraceElement element = ste[2];

            // If the 2nd element in the stack is this class, get the 3rd
            if (element.getClassName().endsWith(LoggingUtil.class.getName())) {
                element = ste[3];
            }
            String className = element.getClassName();

            // remove the package name of the ClassName
            int index = className.lastIndexOf(".");

            if (index > -1) {
                className = className.substring(index + 1, className.length());
            }

            result = className + "." + element.getMethodName() + "()";
        }
        return result;
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> : [CALL ]
     */
    public void call() {
        call(logger);
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message : [CALL ]msg
     * 
     * @param msg
     */
    public void call(Object msg) {
        call(logger, msg);
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message
     * 
     * @param msg
     */
    public void debug(Object message) {
        debug(logger, message);
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> : [ END ]
     */
    public void end() {
        end(logger);
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message : [ END ]msg
     * 
     * @param msg
     */
    public void end(Object msg) {
        end(logger, msg);
    }

    /**
     * if log is enabled, Log as <i>ERROR</i> the message
     * 
     * @param msg
     */
    public void error(Object message) {
        error(logger, message);
    }

    /**
     * if log is enabled, Log as <i>ERROR</i> the exception
     * 
     * @param msg
     * @param throwable
     */
    public void error(Object message, Throwable throwable) {
        error(logger, message, throwable);
    }

    public String getName() {
        return this.logger.getName();
    }

    /**
     * if log is enabled, Log as <i>INFO</i> the message
     * 
     * @param msg
     */
    public void info(Object message) {
        info(logger, message);
    }

    /**
     * if log is enabled, Log as <i>INFO</i> the message and error
     * 
     * @param msg
     * @param error
     */
    public void info(Object message, Throwable error) {
        info(logger, message, error);
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> : [START]
     */
    public void start() {
        start(logger);
    }

    /**
     * if log is enabled, Log as <i>DEBUG</i> the message : [START]msg
     * 
     * @param msg
     */
    public void start(Object msg) {
        start(logger, msg);
    }

    /**
     * if log is enabled, Log as warning the message
     * 
     * @param msg
     */
    public void warning(Object message) {
        warning(logger, message);
    }

    /**
     * if log is enabled, Log as warning the exception
     * 
     * @param msg
     * @param throwable
     */
    public void warning(Object message, Throwable throwable) {
        warning(logger, message, throwable);
    }
}
