/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: NioUtil.java 508 2006-06-02 08:05:25Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.FileChannel;
import java.nio.channels.ReadableByteChannel;

/**
 * @version $Rev: 508 $ $Date: 2006-06-02 10:05:25 +0200 (ven., 02 juin 2006) $
 * @since Petals 1.0
 * @author <a href="mailto:rmarins@fossilec.com">Rafael Marins</a>
 */
public final class NioUtil {

    /**
     * Size of buffer in bytes used during some operations.
     */
    private static final int BUFFER_SIZE = 4096;

    /**
     * Do not construct.
     */
    private NioUtil() {
        super();
    }

    /**
     * Transfer bytes blocks using direct buffer memory allocation. Relies in
     * use the faster Java New I/O instead of traditional byte array move.
     * 
     * @param is
     * @param aFile
     * @throws IOException
     */
    public static void copyStreamToFile(InputStream is, File aFile)
        throws IOException {

        ReadableByteChannel inputChannel = Channels.newChannel(is);

        FileOutputStream fos = new FileOutputStream(aFile);
        FileChannel outputChannel = fos.getChannel();

        ByteBuffer buffer = ByteBuffer.allocate(BUFFER_SIZE);

        int readBytes;
        while ((readBytes = inputChannel.read(buffer)) != -1) {
            if (readBytes > 0) {
                buffer.flip();
                outputChannel.write(buffer);

                buffer.clear();
            }
        }

        try {
            inputChannel.close();
            outputChannel.close();
            fos.close();
        } catch (IOException ioe) {
            // do nothing
        }

    }

}
