/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: PropertyUtil.java 14:47:22 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.util;

import java.net.URL;
import java.util.Properties;

import org.objectweb.petals.PetalsException;

/**
 * TODO the configuration of Joram must be defined with those iinformation,
 * and not use the a3...xml file.
 * HOST parameter has to be defined only once.
 * This class is a util to load properties from a properties file
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public final class PropertyUtil {

    /**
     * Name of the server properties file
     */
    public final static String SERVER_PROPS = "/server.properties";
    
    private PropertyUtil() {
        // Do nothing
    }

    /**
     * Retreive a property from a properties file
     * 
     * @param propertiesFile
     *            name of the properties file (must be in the classpath)
     * @param key
     *            key of the property we want to retreive
     * @return property associated with the key
     * @throws PetalsException
     *             if any problems occurs
     */
    public static String getProperty(String propertiesFile, String key)
        throws PetalsException {
        try {
            URL url = PropertyUtil.class.getResource(propertiesFile);
            Properties properties = new Properties();
            properties.load(url.openStream());
            return properties.getProperty(key);
        } catch (Exception e) {
            throw new PetalsException("Problem while trying to get property "
                + key + " of " + propertiesFile, e);
        }
    }

    /**
     * Retreive the JNDI properties of a properties file
     * 
     * @param propertiesFile
     *            name of the properties file
     * @return bunch of JNDI properties
     * @throws PetalsException
     */
    public static Properties retreiveJNDIProperties(String propertiesFile)
        throws PetalsException {
        try {
            URL url = PropertyUtil.class.getResource(propertiesFile);
            Properties properties = new Properties();
            properties.load(url.openStream());
            Properties outProperties = new Properties();
            outProperties.setProperty("java.naming.factory.host", properties
                .getProperty("java.naming.factory.host"));
            outProperties.setProperty("java.naming.factory.port", properties
                .getProperty("java.naming.factory.port"));
            outProperties.setProperty("java.naming.factory.initial", properties
                .getProperty("java.naming.factory.initial"));
            return outProperties;
        } catch (Exception e) {
            throw new PetalsException(
                "Problem while trying to get JNDI properties of "
                    + propertiesFile, e);
        }
    }
}
